/**
 * This class implements a max heap using an array.  The methods to complete heapsort are included.
 *
 * @author Michael Dube
 * @version 1.0
 * @since 2019-11-01
 */
class MaxHeap {

    /**
     * Creates a MaxHeap using the integers included within orig_values.
     *
     * @param orig_vals Values to sort using heapsort
     */
    MaxHeap(int[] orig_vals) {

        print(orig_vals, "Before Sorting: ");
        // Values within the heap
        int[] values = orig_vals.clone();
        sort(values);
        print(values, "After Sorting: ");

    } // constructor

    /**
     * This method takes in an unsorted array of integers and sorts the array in ascending order
     * using heapsort.
     *
     * @param arr Array to be sorted
     */
    private void sort(int[] arr) {
        int numItems = arr.length;
        int swap;                   // For swapping values within array

        // First, build the heap
        for (int i = numItems / 2 - 1; i >= 0; i--) {
            heapify(arr, numItems, i);
        }

        // Move max (root) to end of array, for each value within the array
        for (int i = numItems - 1; i >= 0; i--) {
            swap = arr[0];
            arr[0] = arr[i];
            arr[i] = swap;

            // Ensure the remainder of the heap satisfies Max Heap Property
            heapify(arr, i, 0); // Only considers arr[0:i]
        }
    } // sort

    /**
     * This method is used to ensure that a subtree rooted with node i, an index within arr[],
     * satisfies the Max Heap Property.  This property dictates that the parent node must be equal
     * to or greater than both children.
     *
     * @param arr Array holding heap
     * @param n   Size of the heap
     * @param i   Root index of subtree being considered
     */
    private void heapify(int[] arr, int n, int i) {
        int maxIndex = i;               // Index of Max Value, initialized as root
        int leftChild = 2 * i + 1;      // Left Child index of Root
        int rightChild = 2 * i + 2;     // Right Child index of Root
        int swap;                       // For swapping values within array

        if (leftChild < n && arr[leftChild] > arr[maxIndex]) { // Left Child Larger than Root
            maxIndex = leftChild;
        }

        if (rightChild < n && arr[rightChild] > arr[maxIndex]) { // Right Child Larger than Root
            maxIndex = rightChild;
        }

        if (maxIndex != i) { // Largest value isn't root (violating max heap property)
            swap = arr[i];
            arr[i] = arr[maxIndex];
            arr[maxIndex] = swap;

            // Check sub-tree
            heapify(arr, n, maxIndex);
        }
    } // heapify

    /**
     * This method prints an array to console with specified beginning message.
     *
     * @param arr Array to print
     * @param msg Message before printed array
     */
    private void print(int[] arr, String msg) {
        System.out.print(msg);
        for (int x : arr) {
            System.out.print(x + "\t");
        }
        System.out.println();
    } // print

} // MinHeap
