import static java.lang.Math.pow;

import java.util.Random;

/**
 * This class implements a word data type in order to store functions needed to sort words
 * alphabetically using heap sort.
 *
 * @author Michael Dube
 * @version 1.0
 * @since 2019-11-01
 */
class Word {

    private static int biggest_word = 0;    // Keeps track of largest generated word
    String spelling;                        // Stores the word

    /**
     * Creates a word with size letters.
     *
     * @param size Number of letters
     */
    Word(int size) {
        spelling = makeWord(size);
        if (size > biggest_word) {
            biggest_word = size;
        }
    } // constructor

    /**
     * Randomly generates a word with size letters and returns the word.
     *
     * @param size Number of letters in generated word.
     * @return Generated word
     */
    private String makeWord(int size) {
        Random rand = new Random();
        StringBuilder rtnVal = new StringBuilder();
        for (int idx = 0; idx < size; idx++) {
            rtnVal.append((char) (rand.nextInt(26) + 'a')); // Appends char ('a' - 'z')
        }
        return rtnVal.toString();
    } // makeWord

    /**
     * Converts a word into a unique numerical value, used to compare words alphabetically.
     *
     * @return int value representing word spelling.
     */
    int wordVal() {
        int wordSum = 0;
        for (int idx = 0; idx < biggest_word; idx++) {
            if (idx < spelling.length()) {
                char c = spelling.charAt(idx);
                wordSum += charVal(c, biggest_word - idx);
            } else {
                wordSum += charVal('a', biggest_word - idx);
            }
        }
        return wordSum;
    } // wordVal

    /**
     * Returns the numerical value of the char c * 26^(place-1).  Basically converting the written
     * word into a base-26 numerical representation such that a word with a greater value is 'later'
     * alphabetically.
     *
     * @param c     char to get value
     * @param place place of char c in original word
     * @return int value of char c
     */
    private int charVal(char c, int place) {
        int loc_in_alpha = c - 'a' + 1;
        return (int) (loc_in_alpha * pow(26, place - 1));
    } // charVal

} // Word
